@@ -680,8 +680,14 @@ class CppCheck(object):
 		path = os.path.dirname(fname)
 		if not os.path.exists(path):
 			os.makedirs(path)
+
 		node = self.bld.path.make_node(fname)
-		node.write(content.encode(encoding))
+		if sys.version_info[0] > 2:
+			if isinstance(content, bytes):
+				content = content.decode(encoding)
+		else:
+			content = content.encode(encoding)
+		node.write(content)
 		return node.abspath().replace('\\', '/')
 
 	def save_xml(self, fname, stderr, cmd):
@@ -711,6 +717,8 @@ class CppCheck(object):
 		lexer = pygments.lexers.guess_lexer_for_filename(source, "")
 		node = self.bld.root.find_node(source)
 		s = node.read()
+		if sys.version_info[0] > 2:
+			s = s.encode()
 		encoding = chardet.detect(s)['encoding']
 		template = Template(CPPCHECK_HTML)
 		context = {}
