@@ -263,123 +263,123 @@ Example below presents the content of the makefile for the *chello* component
 that has been exported from the test directory::
 
 
-		#------------------------------------------------------------------------------
-		# WAFTOOLS generated makefile
-		# version: 0.1.5
-		# waf: 1.7.15
-		#------------------------------------------------------------------------------
+    #------------------------------------------------------------------------------
+    # WAFTOOLS generated makefile
+    # version: 0.1.5
+    # waf: 1.7.15
+    #------------------------------------------------------------------------------
 
-		SHELL=/bin/sh
+    SHELL=/bin/sh
 
-		# commas, spaces and tabs:
-		sp:= 
-		sp+= 
-		tab:=$(sp)$(sp)$(sp)$(sp)
-		comma:=,
+    # commas, spaces and tabs:
+    sp:= 
+    sp+= 
+    tab:=$(sp)$(sp)$(sp)$(sp)
+    comma:=,
 
-		#------------------------------------------------------------------------------
-		# definition of build and install locations
-		#------------------------------------------------------------------------------
-		ifeq ($(TOP),)
-		TOP=$(CURDIR)
-		OUT=$(TOP)/build
-		else
-		OUT=$(subst $(sp),/,$(call rptotop) build $(call rpofcomp))
-		endif
+    #------------------------------------------------------------------------------
+    # definition of build and install locations
+    #------------------------------------------------------------------------------
+    ifeq ($(TOP),)
+    TOP=$(CURDIR)
+    OUT=$(TOP)/build
+    else
+    OUT=$(subst $(sp),/,$(call rptotop) build $(call rpofcomp))
+    endif
 
-		PREFIX?=$(HOME)
-		BINDIR?=$(PREFIX)/bin
-		LIBDIR?=$(PREFIX)/lib
+    PREFIX?=$(HOME)
+    BINDIR?=$(PREFIX)/bin
+    LIBDIR?=$(PREFIX)/lib
 
-		#------------------------------------------------------------------------------
-		# component data
-		#------------------------------------------------------------------------------
-		BIN=chello
-		OUTPUT=$(OUT)/$(BIN)
+    #------------------------------------------------------------------------------
+    # component data
+    #------------------------------------------------------------------------------
+    BIN=chello
+    OUTPUT=$(OUT)/$(BIN)
 
-		# REMARK: use $(wildcard src/*.c) to include all sources.
-		SOURCES= \
-			src/hello.c
+    # REMARK: use $(wildcard src/*.c) to include all sources.
+    SOURCES= \
+        src/hello.c
 
-		OBJECTS=$(SOURCES:.c=.1.o)
+    OBJECTS=$(SOURCES:.c=.1.o)
 
-		DEFINES+=HELLO_VERSION='"1.2.3"'
-		DEFINES:=$(addprefix -D,$(DEFINES))
+    DEFINES+=HELLO_VERSION='"1.2.3"'
+    DEFINES:=$(addprefix -D,$(DEFINES))
 
-		INCLUDES+= \
-			./include
+    INCLUDES+= \
+        ./include
 
-		HEADERS:=$(foreach inc,$(INCLUDES),$(wildcard $(inc)/*.h))
-		INCLUDES:=$(addprefix -I,$(INCLUDES))
+    HEADERS:=$(foreach inc,$(INCLUDES),$(wildcard $(inc)/*.h))
+    INCLUDES:=$(addprefix -I,$(INCLUDES))
 
-		CFLAGS+=
+    CFLAGS+=
 
-		LINKFLAGS+=
+    LINKFLAGS+=
 
-		RPATH+=
-		RPATH:= $(addprefix -Wl$(comma)-rpath$(comma),$(RPATH))
+    RPATH+=
+    RPATH:= $(addprefix -Wl$(comma)-rpath$(comma),$(RPATH))
 
-		LIBPATH_ST+=
-		LIBPATH_ST:= $(addprefix -L,$(LIBPATH_ST))
+    LIBPATH_ST+=
+    LIBPATH_ST:= $(addprefix -L,$(LIBPATH_ST))
 
-		LIB_ST+=
-		LIB_ST:= $(addprefix -l,$(LIB_ST))
+    LIB_ST+=
+    LIB_ST:= $(addprefix -l,$(LIB_ST))
 
-		LIBPATH_SH+=
-		LIBPATH_SH:= $(addprefix -L,$(LIBPATH_SH))
+    LIBPATH_SH+=
+    LIBPATH_SH:= $(addprefix -L,$(LIBPATH_SH))
 
-		LINK_ST= -Wl,-Bstatic $(LIBPATH_ST) $(LIB_ST)
+    LINK_ST= -Wl,-Bstatic $(LIBPATH_ST) $(LIB_ST)
 
-		LIB_SH+=
-		LIB_SH:= $(addprefix -l,$(LIB_SH))
+    LIB_SH+=
+    LIB_SH:= $(addprefix -l,$(LIB_SH))
 
-		LINK_SH= -Wl,-Bdynamic $(LIBPATH_SH) $(LIB_SH)
+    LINK_SH= -Wl,-Bdynamic $(LIBPATH_SH) $(LIB_SH)
 
-		#------------------------------------------------------------------------------
-		# returns the relative path of this component from the top directory
-		#------------------------------------------------------------------------------
-		define rpofcomp
-		$(subst $(subst ~,$(HOME),$(TOP))/,,$(CURDIR))
-		endef
+    #------------------------------------------------------------------------------
+    # returns the relative path of this component from the top directory
+    #------------------------------------------------------------------------------
+    define rpofcomp
+    $(subst $(subst ~,$(HOME),$(TOP))/,,$(CURDIR))
+    endef
 
-		#------------------------------------------------------------------------------
-		# returns the relative path of this component to the top directory
-		#------------------------------------------------------------------------------
-		define rptotop
-		$(foreach word,$(subst /,$(sp),$(call rpofcomp)),..)
-		endef
+    #------------------------------------------------------------------------------
+    # returns the relative path of this component to the top directory
+    #------------------------------------------------------------------------------
+    define rptotop
+    $(foreach word,$(subst /,$(sp),$(call rpofcomp)),..)
+    endef
 
-		#------------------------------------------------------------------------------
-		# define targets
-		#------------------------------------------------------------------------------
-		commands= build clean install uninstall all
+    #------------------------------------------------------------------------------
+    # define targets
+    #------------------------------------------------------------------------------
+    commands= build clean install uninstall all
 
-		.DEFAULT_GOAL=all
+    .DEFAULT_GOAL=all
 
-		#------------------------------------------------------------------------------
-		# definitions of recipes (i.e. make targets)
-		#------------------------------------------------------------------------------
-		all: build
+    #------------------------------------------------------------------------------
+    # definitions of recipes (i.e. make targets)
+    #------------------------------------------------------------------------------
+    all: build
 
-		build: $(OBJECTS)
-			$(CC) $(LINKFLAGS) $(addprefix $(OUT)/,$(OBJECTS)) -o $(OUTPUT) $(RPATH) $(LINK_ST) $(LINK_SH)
+    build: $(OBJECTS)
+        $(CC) $(LINKFLAGS) $(addprefix $(OUT)/,$(OBJECTS)) -o $(OUTPUT) $(RPATH) $(LINK_ST) $(LINK_SH)
 
-		clean:
-			$(foreach obj,$(OBJECTS),rm -f $(OUT)/$(obj);)
-			rm -f $(OUTPUT)
+    clean:
+        $(foreach obj,$(OBJECTS),rm -f $(OUT)/$(obj);)
+        rm -f $(OUTPUT)
 
-		install: build
-			mkdir -p $(BINDIR)
-			cp $(OUTPUT) $(BINDIR)
+    install: build
+        mkdir -p $(BINDIR)
+        cp $(OUTPUT) $(BINDIR)
 
-		uninstall:
-			rm -f $(BINDIR)/$(BIN)
+    uninstall:
+        rm -f $(BINDIR)/$(BIN)
 
-		$(OBJECTS): $(HEADERS)
-			mkdir -p $(OUT)/$(dir $@)
-			$(CC) $(CFLAGS) $(INCLUDES) $(DEFINES) $(subst .1.o,.c,$@) -c -o $(OUT)/$@
+    $(OBJECTS): $(HEADERS)
+        mkdir -p $(OUT)/$(dir $@)
+        $(CC) $(CFLAGS) $(INCLUDES) $(DEFINES) $(subst .1.o,.c,$@) -c -o $(OUT)/$@
 
-		.PHONY: $(commands)
+    .PHONY: $(commands)
 
 
 Note that only one segment of this file, the named *component* *data* contains 
@@ -424,6 +424,7 @@ import re
 from waflib import Utils, Context, Logs
 from waflib.Build import BuildContext
 import waftools
+from waftools import deps
 
 
 def options(opt):
@@ -489,10 +490,15 @@ def export(bld):
 		return
 
 	root = MakeRoot(bld)
+	targets = waftools.deps.get_targets(bld)
+	
 	for tgen in bld.task_gen_cache_names.values():
-		child = MakeChild(bld, tgen, tgen.tasks)
-		child.export()
-		root.add_child(child.get_data())
+		if targets and tgen.get_name() not in targets:
+			continue
+		if set(('c', 'cxx')) & set(tgen.features):
+			child = MakeChild(bld, tgen, tgen.tasks)
+			child.export()
+			root.add_child(child.get_data())
 	root.export()
 
 
@@ -506,9 +512,15 @@ def cleanup(bld):
 		return
 
 	root = MakeRoot(bld)
+	targets = waftools.deps.get_targets(bld)
+	
 	for tgen in bld.task_gen_cache_names.values():
-		child = MakeChild(bld, tgen, tgen.tasks)
-		child.cleanup()
+		if targets and tgen.get_name() not in targets:
+			continue
+		if set(('c', 'cxx')) & set(tgen.features):
+			child = MakeChild(bld, tgen, tgen.tasks)
+			child.cleanup()
+	
 	root.cleanup()
 
 
@@ -517,29 +529,29 @@ class Make(object):
 		self.bld = bld
 
 	def export(self):
-		content = self._get_content()
+		content = self.get_content()
 		if not content:
 			return
-		node = self._make_node()
+		node = self.make_node()
 		if not node:
 			return
 		node.write(content)
 		Logs.pprint('YELLOW', 'exported: %s' % node.abspath())
 			
 	def cleanup(self):
-		node = self._find_node()
+		node = self.find_node()
 		if node:
 			node.delete()
 			Logs.pprint('YELLOW', 'removed: %s' % node.abspath())
 
-	def _find_node(self):
-		name = self._get_name()
+	def find_node(self):
+		name = self.get_name()
 		if not name:
 			return None
 		return self.bld.srcnode.find_node(name)
 
-	def _make_node(self):
-		name = self._get_name()
+	def make_node(self):
+		name = self.get_name()
 		if not name:
 			return None
 		return self.bld.srcnode.make_node(name)
@@ -550,11 +562,11 @@ class Make(object):
 		s = re.sub('==VERSION==', waftools.version, s)
 		return s
 
-	def _get_name(self):
+	def get_name(self):
 		'''abstract operation to be defined in child'''
 		return None
 
-	def _get_content(self):
+	def get_content(self):
 		'''abstract operation to be defined in child'''
 		return None
 
@@ -564,15 +576,15 @@ class MakeRoot(Make):
 		super(MakeRoot, self).__init__(bld)
 		self.childs = []
 
-	def _get_name(self):
-		name = self.bld.path.relpath().replace('\\', '/')		
+	def get_name(self):
+		name = self.bld.path.relpath().replace('\\', '/')
 		if self.bld.variant:
 			name += '/makefile_%s.mk' % (self.bld.variant)
 		else:
 			name += '/Makefile'
 		return name
 
-	def _get_content(self):
+	def get_content(self):
 		bld = self.bld
 		cwd = str(os.getcwd()).replace('\\', '/')
 		prefix = str(os.path.abspath(bld.env.PREFIX)).replace('\\', '/')
@@ -599,7 +611,8 @@ class MakeRoot(Make):
 		except IndexError:
 			cxx = 'g++'
 		cxx = cxx.replace('\\', '/')
-
+	
+		var = self.bld.variant
 		s = MAKEFILE_ROOT
 		s = super(MakeRoot, self).populate(s)
 		s = re.sub('APPNAME:=', 'APPNAME:=%s' % getattr(Context.g_module, Context.APPNAME), s)
@@ -614,9 +627,10 @@ class MakeRoot(Make):
 		s = re.sub('CFLAGS:=', 'CFLAGS:=%s' % ' '.join(bld.env.CFLAGS), s)
 		s = re.sub('CXXFLAGS:=', 'CXXFLAGS:=%s' % ' '.join(bld.env.CXXFLAGS), s)
 		s = re.sub('DEFINES:=', 'DEFINES:=%s' % ' '.join(bld.env.DEFINES), s)
-		s = re.sub('==MODULES==', self._get_modules(), s)
-		s = re.sub('==MODPATHS==', self._get_modpaths(), s)
-		s = re.sub('==MODDEPS==', self._get_moddeps(), s)
+		s = re.sub('==MODULES==', self.get_modules(), s)
+		s = re.sub('==MODPATHS==', self.get_modpaths(), s)
+		s = re.sub('==MODDEPS==', self.get_moddeps(), s)
+		s = re.sub('==VARIANT==', '_%s' % var if var else '', s)
 		s = re.sub(bld.env.PREFIX, '$(PREFIX)', s)
 		s = re.sub('PREFIX:=', 'PREFIX:=%s' % prefix, s)
 		return s
@@ -624,14 +638,14 @@ class MakeRoot(Make):
 	def add_child(self, child):
 		self.childs.append(child)
 	
-	def _get_modules(self):
+	def get_modules(self):
 		d = []
 		for (name, _, _) in self.childs:
 			d.append(name)
 		s = ' \\\n\t'.join(d)
 		return s
 
-	def _get_modpaths(self):
+	def get_modpaths(self):
 		d = []
 		for (name, makefile, _) in self.childs:
 			s = '%s;%s' %(name, os.path.dirname(makefile))
@@ -639,12 +653,14 @@ class MakeRoot(Make):
 		s = ' \\\n\t'.join(d)
 		return s
 
-	def _get_moddeps(self):
+	def get_moddeps(self):
 		d = []
+		names = [c[0] for c in self.childs]
 		for (name, _, deps) in self.childs:
 			s = '%s;' %(name)
-			if len(deps):
-				s += ','.join(deps)
+			for dep in deps:
+				if dep in names:
+					s += ',%s' % dep
 			d.append(s)
 		s = ' \\\n\t'.join(d)
 		return s
@@ -655,62 +671,105 @@ class MakeChild(Make):
 		super(MakeChild, self).__init__(bld)
 		self.gen = gen
 		self.targets = targets
-		self._process()
+		self.process()
 
-	def _get_name(self):
+	def get_name(self):
 		gen = self.gen
 		if self.bld.variant:
 			name = '%s/%s_%s.mk' % (gen.path.relpath(), gen.get_name(), self.bld.variant)
 		else:
-			name = '%s/%s.mk' % (gen.path.relpath(), gen.get_name())			
+			name = '%s/%s.mk' % (gen.path.relpath(), gen.get_name())
 		return name.replace('\\', '/')
 
-	def _get_content(self):
-		if 'cprogram' in self.gen.features:
-			return self._get_cprogram_content()
-		elif 'cstlib' in self.gen.features:
-			return self._get_cstlib_content()
-		elif 'cshlib' in self.gen.features:
-			return self._get_cshlib_content()
-		elif 'cxxprogram' in self.gen.features:
-			return self._get_cxxprogram_content()
-		elif 'cxxstlib' in self.gen.features:
-			return self._get_cxxstlib_content()
-		elif 'cxxshlib' in self.gen.features:
-			return self._get_cxxshlib_content()
+	def get_content(self):
+		features = list(self.gen.features)
+		if features == ['cxx']:
+			features.append('cxxstlib')
+		elif features == ['c']:
+			features.append('cstlib')
 
-		s = MAKEFILE_CHILD
-		s = super(MakeChild, self).populate(s)
-		return s
+		s = ''
+		for feature in features:
+			if hasattr(self, feature):
+				func = getattr(self, feature)
+				s += func(self.bld, self.gen)
+				break
+		
+		if s == '':
+			self.bld.fatal('FAILED(%s) features=%s' % (self.gen.get_name(), self.gen.features))
+
+		loc = 'build'
+		if self.bld.variant:
+			loc += '/%s' % self.bld.variant
+		install = MAKEFILE_LOC % (loc, loc)
+		return re.sub('==INSTALL==', install, s)
 		
 	def get_data(self):
 		gen = self.gen
 		name = gen.get_name()
-		makefile = self._get_name()
+		makefile = self.get_name()
 		deps = Utils.to_list(getattr(gen, 'use', []))
 		return (name, makefile, deps)
-		
-	def _process(self):
+	
+	def get_libs(self, target, nest):
+		'''returns library information for this target and any target it depends on
+
+		returns a list of tuples;
+			(nest, name, paths, kind)
+
+			nest: nesting level (int)
+			name: library name (str)
+			paths: list of search paths or None
+			kind: library type ('shared' OR 'static')
+		'''
+		try:
+			tg = self.bld.get_tgen_by_name(target)
+		except Errors.WafError:
+			return []
+		name = tg.get_name()
+		features = Utils.to_list(getattr(tg, 'features', []))
+		if 'fake_lib' in features:
+			paths = [p.replace('\\', '/') for p in tg.lib_paths]
+			return [(nest, name, paths, 'shared')]
+		if not set(('c','cxx')) & set(features):
+			return []
+		paths = [tg.path.relpath().replace('\\','/')]
+		if set(('cshlib', 'cxxshlib')) & set(features):
+			return [(nest, name, paths, 'shared')]
+		libs = [(nest, name, paths, 'static')]
+		for lib in Utils.to_list(getattr(tg, 'lib', [])):
+			libs.append((nest, lib, None, 'shared'))
+		for use in getattr(tg, 'use', []):
+			libs.extend(self.get_libs(use, nest+1))
+		return libs
+
+	def process(self):
 		self.lib = {}
 		self.lib['static'] = { 'name' : [], 'path' : [] }
 		self.lib['shared'] = { 'name' : [], 'path' : [] }
+
+		# construct list of (lib) dependencies and sort by task
+		# with highest nesting
+		uses = []
 		for use in Utils.to_list(getattr(self.gen, 'use', [])):
-			tg = self.bld.get_tgen_by_name(use)
-			if len(set(tg.features) & set(['cstlib', 'cxxstlib'])):
-				self.lib['static']['name'].append(tg.get_name())
-				self.lib['static']['path'].append(tg.path.relpath().replace('\\','/'))
-			if len(set(tg.features) & set(['cshlib', 'cxxshlib'])):
-				self.lib['shared']['name'].append(tg.get_name())
-				self.lib['shared']['path'].append(tg.path.relpath().replace('\\','/'))
+			uses.extend(self.get_libs(use, 0))
+		uses = sorted(uses, key=lambda use: use[0])
+		uses = uses[::-1]
+
+		# insert tasks at start of list, lowest nesting at start
+		for (_, lib, paths, kind) in uses:
+			self.lib[kind]['name'].insert(0, lib)
+			if paths:
+				self.lib[kind]['path'].extend(paths)
 		for lib in Utils.to_list(getattr(self.gen, 'lib', [])):
 			self.lib['shared']['name'].append(lib)
 
-	def _get_cprogram_content(self):
-		bld = self.bld
-		gen = self.gen
-		source = self._get_genlist(gen, 'source')
-		includes = self._get_genlist(gen, 'includes')
-		defines = self._get_defines(gen)
+	def cprogram(self, bld, gen):
+		'''returns the makefile content for a C program.
+		'''
+		source = self.get_genlist(gen, 'source')
+		includes = self.get_genlist(gen, 'includes')
+		defines = self.get_defines(gen)
 		defines = [d for d in defines]
 		s = MAKEFILE_CPROGRAM
 		s = super(MakeChild, self).populate(s)
@@ -719,20 +778,20 @@ class MakeChild(Make):
 		s = re.sub('SOURCES=', 'SOURCES= \\\n\t%s' % ' \\\n\t'.join(source), s)
 		s = re.sub('INCLUDES\+=', 'INCLUDES+= \\\n\t%s' % ' \\\n\t'.join(includes),s)
 		s = re.sub('DEFINES\+=', 'DEFINES+=%s' % ' '.join(defines),s)
-		s = re.sub('CFLAGS\+=', 'CFLAGS+=%s' % self._get_cflags(gen),s)
-		s = re.sub('LINKFLAGS\+=', 'LINKFLAGS+=%s' % self._get_linkflags(gen),s)
-		s = re.sub('LIBPATH_ST\+=', 'LIBPATH_ST+=%s' % self._get_libpath('static'),s)
-		s = re.sub('LIB_ST\+=', 'LIB_ST+=%s' % self._get_lib('static'),s)
-		s = re.sub('LIBPATH_SH\+=', 'LIBPATH_SH+=%s' % self._get_libpath('shared'),s)
-		s = re.sub('LIB_SH\+=', 'LIB_SH+=%s' % self._get_lib('shared'),s)
+		s = re.sub('CFLAGS\+=', 'CFLAGS+=%s' % self.get_cflags(gen),s)
+		s = re.sub('LINKFLAGS\+=', 'LINKFLAGS+=%s' % self.get_linkflags(gen),s)
+		s = re.sub('LIBPATH_ST\+=', 'LIBPATH_ST+=%s' % self.get_libpath('static'),s)
+		s = re.sub('LIB_ST\+=', 'LIB_ST+=%s' % self.get_lib('static'),s)
+		s = re.sub('LIBPATH_SH\+=', 'LIBPATH_SH+=%s' % self.get_libpath('shared'),s)
+		s = re.sub('LIB_SH\+=', 'LIB_SH+=%s' % self.get_lib('shared'),s)
 		return s
 
-	def _get_cxxprogram_content(self):
-		bld = self.bld
-		gen = self.gen
-		source = self._get_genlist(gen, 'source')
-		includes = self._get_genlist(gen, 'includes')
-		defines = self._get_defines(gen)
+	def cxxprogram(self, bld, gen):
+		'''returns the makefile content for a C++ program.
+		'''
+		source = self.get_genlist(gen, 'source')
+		includes = self.get_genlist(gen, 'includes')
+		defines = self.get_defines(gen)
 		defines = [d for d in defines]
 		s = MAKEFILE_CXXPROGRAM
 		s = super(MakeChild, self).populate(s)
@@ -741,20 +800,20 @@ class MakeChild(Make):
 		s = re.sub('SOURCES=', 'SOURCES= \\\n\t%s' % ' \\\n\t'.join(source), s)
 		s = re.sub('INCLUDES\+=', 'INCLUDES+= \\\n\t%s' % ' \\\n\t'.join(includes),s)
 		s = re.sub('DEFINES\+=', 'DEFINES+=%s' % ' '.join(defines),s)
-		s = re.sub('CXXFLAGS\+=', 'CXXFLAGS+=%s' % self._get_cxxflags(gen),s)
-		s = re.sub('LINKFLAGS\+=', 'LINKFLAGS+=%s' % self._get_linkflags(gen),s)
-		s = re.sub('LIBPATH_ST\+=', 'LIBPATH_ST+=%s' % self._get_libpath('static'),s)
-		s = re.sub('LIB_ST\+=', 'LIB_ST+=%s' % self._get_lib('static'),s)
-		s = re.sub('LIBPATH_SH\+=', 'LIBPATH_SH+=%s' % self._get_libpath('shared'),s)
-		s = re.sub('LIB_SH\+=', 'LIB_SH+=%s' % self._get_lib('shared'),s)
+		s = re.sub('CXXFLAGS\+=', 'CXXFLAGS+=%s' % self.get_cxxflags(gen),s)
+		s = re.sub('LINKFLAGS\+=', 'LINKFLAGS+=%s' % self.get_linkflags(gen),s)
+		s = re.sub('LIBPATH_ST\+=', 'LIBPATH_ST+=%s' % self.get_libpath('static'),s)
+		s = re.sub('LIB_ST\+=', 'LIB_ST+=%s' % self.get_lib('static'),s)
+		s = re.sub('LIBPATH_SH\+=', 'LIBPATH_SH+=%s' % self.get_libpath('shared'),s)
+		s = re.sub('LIB_SH\+=', 'LIB_SH+=%s' % self.get_lib('shared'),s)
 		return s
 
-	def _get_cstlib_content(self):
-		bld = self.bld
-		gen = self.gen
-		source = self._get_genlist(gen, 'source')
-		includes = self._get_genlist(gen, 'includes')
-		defines = self._get_defines(gen)
+	def cstlib(self, bld, gen):
+		'''returns the makefile content for a C static library.
+		'''
+		source = self.get_genlist(gen, 'source')
+		includes = self.get_genlist(gen, 'includes')
+		defines = self.get_defines(gen)
 		defines = [d for d in defines]
 		s = MAKEFILE_CSTLIB
 		s = super(MakeChild, self).populate(s)
@@ -763,16 +822,16 @@ class MakeChild(Make):
 		s = re.sub('SOURCES=', 'SOURCES= \\\n\t%s' % ' \\\n\t'.join(source), s)
 		s = re.sub('INCLUDES\+=', 'INCLUDES+= \\\n\t%s' % ' \\\n\t'.join(includes),s)
 		s = re.sub('DEFINES\+=', 'DEFINES+=%s' % ' '.join(defines),s)
-		s = re.sub('CFLAGS\+=', 'CFLAGS+=%s' % self._get_cflags(gen),s)
-		s = re.sub('ARFLAGS=', 'ARFLAGS=%s' % bld.env.ARFLAGS, s)
+		s = re.sub('CFLAGS\+=', 'CFLAGS+=%s' % self.get_cflags(gen),s)
+		s = re.sub('ARFLAGS=', 'ARFLAGS=%s' % self.get_arflags(gen), s)
 		return s
 
-	def _get_cshlib_content(self):
-		bld = self.bld
-		gen = self.gen
-		source = self._get_genlist(gen, 'source')
-		includes = self._get_genlist(gen, 'includes')
-		defines = self._get_defines(gen)
+	def cshlib(self, bld, gen):
+		'''returns the makefile content for a C shared library.
+		'''
+		source = self.get_genlist(gen, 'source')
+		includes = self.get_genlist(gen, 'includes')
+		defines = self.get_defines(gen)
 		defines = [d for d in defines]
 		s = MAKEFILE_CSHLIB
 		s = super(MakeChild, self).populate(s)
@@ -783,20 +842,20 @@ class MakeChild(Make):
 		s = re.sub('SOURCES=', 'SOURCES= \\\n\t%s' % ' \\\n\t'.join(source), s)
 		s = re.sub('INCLUDES\+=', 'INCLUDES+= \\\n\t%s' % ' \\\n\t'.join(includes),s)
 		s = re.sub('DEFINES\+=', 'DEFINES+=%s' % ' '.join(defines),s)
-		s = re.sub('CFLAGS\+=', 'CFLAGS+=%s' % self._get_cflags(gen),s)
-		s = re.sub('LINKFLAGS\+=', 'LINKFLAGS+=%s' % self._get_linkflags(gen),s)
-		s = re.sub('LIBPATH_ST\+=', 'LIBPATH_ST+=%s' % self._get_libpath('static'),s)
-		s = re.sub('LIB_ST\+=', 'LIB_ST+=%s' % self._get_lib('static'),s)
-		s = re.sub('LIBPATH_SH\+=', 'LIBPATH_SH+=%s' % self._get_libpath('shared'),s)
-		s = re.sub('LIB_SH\+=', 'LIB_SH+=%s' % self._get_lib('shared'),s)
+		s = re.sub('CFLAGS\+=', 'CFLAGS+=%s' % self.get_cflags(gen),s)
+		s = re.sub('LINKFLAGS\+=', 'LINKFLAGS+=%s' % self.get_linkflags(gen),s)
+		s = re.sub('LIBPATH_ST\+=', 'LIBPATH_ST+=%s' % self.get_libpath('static'),s)
+		s = re.sub('LIB_ST\+=', 'LIB_ST+=%s' % self.get_lib('static'),s)
+		s = re.sub('LIBPATH_SH\+=', 'LIBPATH_SH+=%s' % self.get_libpath('shared'),s)
+		s = re.sub('LIB_SH\+=', 'LIB_SH+=%s' % self.get_lib('shared'),s)
 		return s
 
-	def _get_cxxstlib_content(self):
-		bld = self.bld
-		gen = self.gen
-		source = self._get_genlist(gen, 'source')
-		includes = self._get_genlist(gen, 'includes')
-		defines = self._get_defines(gen)
+	def cxxstlib(self, bld, gen):
+		'''returns the makefile content for a C++ static library.
+		'''
+		source = self.get_genlist(gen, 'source')
+		includes = self.get_genlist(gen, 'includes')
+		defines = self.get_defines(gen)
 		defines = [d for d in defines]
 		s = MAKEFILE_CXXSTLIB
 		s = super(MakeChild, self).populate(s)
@@ -805,16 +864,16 @@ class MakeChild(Make):
 		s = re.sub('SOURCES=', 'SOURCES= \\\n\t%s' % ' \\\n\t'.join(source), s)
 		s = re.sub('INCLUDES\+=', 'INCLUDES+= \\\n\t%s' % ' \\\n\t'.join(includes),s)
 		s = re.sub('DEFINES\+=', 'DEFINES+=%s' % ' '.join(defines),s)
-		s = re.sub('CXXFLAGS\+=', 'CXXFLAGS+=%s' % self._get_cxxflags(gen),s)
-		s = re.sub('ARFLAGS=', 'ARFLAGS=%s' % bld.env.ARFLAGS, s)
+		s = re.sub('CXXFLAGS\+=', 'CXXFLAGS+=%s' % self.get_cxxflags(gen),s)
+		s = re.sub('ARFLAGS=', 'ARFLAGS=%s' % self.get_arflags(gen), s)
 		return s
 
-	def _get_cxxshlib_content(self):
-		bld = self.bld
-		gen = self.gen
-		source = self._get_genlist(gen, 'source')
-		includes = self._get_genlist(gen, 'includes')
-		defines = self._get_defines(gen)
+	def cxxshlib(self, bld, gen):
+		'''returns the makefile content for a C++ shared library.
+		'''
+		source = self.get_genlist(gen, 'source')
+		includes = self.get_genlist(gen, 'includes')
+		defines = self.get_defines(gen)
 		defines = [d for d in defines]
 		s = MAKEFILE_CXXSHLIB
 		s = super(MakeChild, self).populate(s)
@@ -825,22 +884,22 @@ class MakeChild(Make):
 		s = re.sub('SOURCES=', 'SOURCES= \\\n\t%s' % ' \\\n\t'.join(source), s)
 		s = re.sub('INCLUDES\+=', 'INCLUDES+= \\\n\t%s' % ' \\\n\t'.join(includes),s)
 		s = re.sub('DEFINES\+=', 'DEFINES+=%s' % ' '.join(defines),s)
-		s = re.sub('CXXFLAGS\+=', 'CXXFLAGS+=%s' % self._get_cxxflags(gen),s)
-		s = re.sub('LINKFLAGS\+=', 'LINKFLAGS+=%s' % self._get_linkflags(gen),s)
-		s = re.sub('LIBPATH_ST\+=', 'LIBPATH_ST+=%s' % self._get_libpath('static'),s)
-		s = re.sub('LIB_ST\+=', 'LIB_ST+=%s' % self._get_lib('static'),s)
-		s = re.sub('LIBPATH_SH\+=', 'LIBPATH_SH+=%s' % self._get_libpath('shared'),s)
-		s = re.sub('LIB_SH\+=', 'LIB_SH+=%s' % self._get_lib('shared'),s)
+		s = re.sub('CXXFLAGS\+=', 'CXXFLAGS+=%s' % self.get_cxxflags(gen),s)
+		s = re.sub('LINKFLAGS\+=', 'LINKFLAGS+=%s' % self.get_linkflags(gen),s)
+		s = re.sub('LIBPATH_ST\+=', 'LIBPATH_ST+=%s' % self.get_libpath('static'),s)
+		s = re.sub('LIB_ST\+=', 'LIB_ST+=%s' % self.get_lib('static'),s)
+		s = re.sub('LIBPATH_SH\+=', 'LIBPATH_SH+=%s' % self.get_libpath('shared'),s)
+		s = re.sub('LIB_SH\+=', 'LIB_SH+=%s' % self.get_lib('shared'),s)
 		return s
 
-	def _get_genlist(self, gen, name):
+	def get_genlist(self, gen, name):
 		lst = Utils.to_list(getattr(gen, name, []))
 		lst = [str(l.path_from(gen.path)) if hasattr(l, 'path_from') else l for l in lst]
 		return [l.replace('\\', '/') for l in lst]
 
-	def _get_defines(self, gen):
+	def get_defines(self, gen):
 		defines = []
-		defs = self._get_genlist(gen, 'defines')
+		defs = self.get_genlist(gen, 'defines')
 		for d in defs:
 			if d.count('"') == 2:
 				(pre, val, post) = d.split('"')
@@ -848,19 +907,23 @@ class MakeChild(Make):
 			defines.append(d)
 		return defines
 
-	def _get_cflags(self, gen):
+	def get_cflags(self, gen):
 		cflags = getattr(gen, 'cflags', [])
+		if isinstance(cflags, str):
+			cflags = cflags.split()
 		if 'cshlib' in gen.features:
 			cflags.extend(self.bld.env.CFLAGS_cshlib)
 		return ' '.join(cflags)
 
-	def _get_cxxflags(self, gen):
+	def get_cxxflags(self, gen):
 		cxxflags = getattr(gen, 'cxxflags', [])
+		if isinstance(cxxflags, str):
+			cxxflags = cxxflags.split()
 		if 'cxxshlib' in gen.features:
 			cxxflags.extend(self.bld.env.CXXFLAGS_cxxshlib)
 		return ' '.join(cxxflags)
 
-	def _get_linkflags(self, gen):
+	def get_linkflags(self, gen):
 		linkflags = getattr(gen, 'linkflags', [])
 		if 'cshlib' in gen.features:
 			linkflags.extend(self.bld.env.LINKFLAGS_cshlib)
@@ -868,11 +931,24 @@ class MakeChild(Make):
 			linkflags.extend(self.bld.env.LINKFLAGS_cxxshlib)
 		return ' '.join(linkflags)
 
-	def _get_libpath(self, kind):
-		libpath = self.lib[kind]['path']
-		return ' \\\n\t'.join(['$(TOP)/build/%s' % l for l in libpath])
+	def get_arflags(self, gen):
+		flags = Utils.to_list(self.bld.env.ARFLAGS)
+		return ' '.join(flags)
 
-	def _get_lib(self, kind):
+	def get_libpath(self, kind):
+		paths = self.lib[kind]['path']
+		top = self.bld.path.abspath().replace('\\', '/')
+		libpath = []
+		for p in paths:
+			if os.path.abspath(p) and p.startswith(top):
+				libpath.append(p.replace(top, '$(TOP)', 1))
+			elif self.bld.variant:
+				libpath.append('$(TOP)/build/%s/%s' % (self.bld.variant, p))
+			else:
+				libpath.append('$(TOP)/build/%s' % p)
+		return ' \\\n\t'.join(libpath)
+
+	def get_lib(self, kind):
 		lib = self.lib[kind]['name']
 		return ' '.join(lib)
 
@@ -989,7 +1065,7 @@ endef
 #------------------------------------------------------------------------------
 define domake
 $1: $(call getdeps, $1)
-	$(MAKE) -r -C $(call getpath,$1) -f $(lastword $(subst $(csep),$(sp),$1)).mk $(firstword $(subst $(csep),$(sp),$1))
+	$(MAKE) -r -C $(call getpath,$1) -f $(lastword $(subst $(csep),$(sp),$1))==VARIANT==.mk $(firstword $(subst $(csep),$(sp),$1))
 endef
 
 #------------------------------------------------------------------------------
@@ -1043,13 +1119,13 @@ help:
 	@echo "commands:"
 	@echo "  all                                 builds all modules"
 	@echo "  build                               builds all modules"
-	@echo "  build$(csep)a                             builds module 'a' and it's dependencies"
+	@echo "  build$(csep)a                       builds module 'a' and it's dependencies"
 	@echo "  clean                               removes all build intermediates and outputs"
-	@echo "  clean$(csep)a                             cleans module 'a' and it's dependencies"
+	@echo "  clean$(csep)a                       cleans module 'a' and it's dependencies"
 	@echo "  install                             installs files in $(PREFIX)"
-	@echo "  install$(csep)a                           installs module 'a' and it's dependencies"
+	@echo "  install$(csep)a                     installs module 'a' and it's dependencies"
 	@echo "  uninstall                           removes all installed files from $(PREFIX)"
-	@echo "  uninstall$(csep)a                         removes module 'a' and it's dependencies"
+	@echo "  uninstall$(csep)a                   removes module 'a' and it's dependencies"
 	@echo "  list                                list available make commands (i.e. recipes)"
 	@echo "  modules                             list logical names of all modules"
 	@echo "  find [SEARCHPATH=] [SEARCHFILE=]    searches for files default(path=$(SEARCHPATH),file=$(SEARCHFILE))"
@@ -1099,6 +1175,16 @@ uninstall:
 
 '''
 
+MAKEFILE_LOC = \
+'''
+ifeq ($(TOP),)
+TOP=$(CURDIR)
+OUT=$(TOP)/%s
+else
+OUT=$(subst $(sp),/,$(call rptotop) %s $(call rpofcomp))
+endif
+'''
+
 MAKEFILE_CPROGRAM = \
 '''#------------------------------------------------------------------------------
 # WAFTOOLS generated makefile
@@ -1117,12 +1203,7 @@ comma:=,
 #------------------------------------------------------------------------------
 # definition of build and install locations
 #------------------------------------------------------------------------------
-ifeq ($(TOP),)
-TOP=$(CURDIR)
-OUT=$(TOP)/build
-else
-OUT=$(subst $(sp),/,$(call rptotop) build $(call rpofcomp))
-endif
+==INSTALL==
 
 PREFIX?=$(HOME)
 BINDIR?=$(PREFIX)/bin
@@ -1236,12 +1317,7 @@ comma:=,
 #------------------------------------------------------------------------------
 # definition of build and install locations
 #------------------------------------------------------------------------------
-ifeq ($(TOP),)
-TOP=$(CURDIR)
-OUT=$(TOP)/build
-else
-OUT=$(subst $(sp),/,$(call rptotop) build $(call rpofcomp))
-endif
+==INSTALL==
 
 PREFIX?=$(HOME)
 BINDIR?=$(PREFIX)/bin
@@ -1354,12 +1430,7 @@ comma:=,
 #------------------------------------------------------------------------------
 # definition of build and install locations
 #------------------------------------------------------------------------------
-ifeq ($(TOP),)
-TOP=$(CURDIR)
-OUT=$(TOP)/build
-else
-OUT=$(subst $(sp),/,$(call rptotop) build $(call rpofcomp))
-endif
+==INSTALL==
 
 #------------------------------------------------------------------------------
 # component data
@@ -1447,12 +1518,7 @@ comma:=,
 #------------------------------------------------------------------------------
 # definition of build and install locations
 #------------------------------------------------------------------------------
-ifeq ($(TOP),)
-TOP=$(CURDIR)
-OUT=$(TOP)/build
-else
-OUT=$(subst $(sp),/,$(call rptotop) build $(call rpofcomp))
-endif
+==INSTALL==
 
 PREFIX?=$(HOME)
 LIBDIR?=$(PREFIX)/lib
@@ -1573,12 +1639,7 @@ comma:=,
 #------------------------------------------------------------------------------
 # definition of build and install locations
 #------------------------------------------------------------------------------
-ifeq ($(TOP),)
-TOP=$(CURDIR)
-OUT=$(TOP)/build
-else
-OUT=$(subst $(sp),/,$(call rptotop) build $(call rpofcomp))
-endif
+==INSTALL==
 
 PREFIX?=$(HOME)
 LIBDIR?=$(PREFIX)/lib
@@ -1699,12 +1760,7 @@ comma:=,
 #------------------------------------------------------------------------------
 # definition of build and install locations
 #------------------------------------------------------------------------------
-ifeq ($(TOP),)
-TOP=$(CURDIR)
-OUT=$(TOP)/build
-else
-OUT=$(subst $(sp),/,$(call rptotop) build $(call rpofcomp))
-endif
+==INSTALL==
 
 #------------------------------------------------------------------------------
 # component data
